use engine::prelude::*;

use crate::components::attributes::Attribute;

create_settings_section!(
    AgilitySettings,
    "Agility",
    {
        armor: u32,
        crit_chance: f32,
        crit_damage: f32,
    }
);

create_settings_section!(
    StrengthSettings,
    "Strength",
    {
        physical_damage: u32,
        air_resistance: u32,
        fire_resistance: u32,
        water_resistance: u32,
        health: f32,
        health_regen: f32,
    }
);

create_settings_section!(
    IntelligenceSettings,
    "Intelligence",
    {
        air_damage: u32,
        fire_damage: u32,
        water_damage: u32,
        mana: f32,
        mana_regen: f32,
    }
);

create_settings_section!(
    MetaSection,
    "Meta",
    {
        starting_skill_points: u32,
        skill_points_per_level: u32,
    }
);

create_settings_section!(
    StartingAttributes,
    "StartingAttributes",
    {
        strength: u32,
        agility: u32,
        intelligence: u32,
    }
);

create_settings_section!(
    AttributeColorSettings,
    "AttributeColorSettings",
    {
        strength: Color,
        agility: Color,
        intelligence: Color,
    }
);

impl AttributeColorSettings {
    pub fn from_attribute(&self, attribute: Attribute) -> Color {
        match attribute {
            Attribute::Agility => self.agility,
            Attribute::Intelligence => self.intelligence,
            Attribute::Strength => self.strength,
        }
    }
}

impl StartingAttributes {
    pub fn sum(&self) -> u32 {
        self.strength + self.agility + self.intelligence
    }
}

create_settings_container!(
    AttributeSettings,
    {
        meta_settings: MetaSection,
        agility_settings: AgilitySettings,
        strength_settings: StrengthSettings,
        intelligence_settings: IntelligenceSettings,
        starting_attributes: StartingAttributes,
        attribute_color_settings: AttributeColorSettings,
    }
);
